import * as React from 'react';
import { Align, VariableSizeListProps } from 'react-window';
import { CommonNodeComponentProps, CommonNodeData, CommonNodeRecord, CommonUpdateOptions, TreeProps, TreeState } from './utils';
export declare type VariableSizeUpdateOptions = CommonUpdateOptions & {
    readonly useDefaultHeight?: boolean;
};
export declare type VariableSizeNodeData<T> = {
    /** Default node height. Can be used only with VariableSizeTree */
    readonly defaultHeight: number;
} & CommonNodeData<T>;
export declare type VariableSizeNodeComponentProps<T> = CommonNodeComponentProps<VariableSizeNodeData<T>, T> & {
    readonly height: number;
    readonly resize: (height: number, shouldForceUpdate?: boolean) => void;
};
export declare type VariableSizeNodeRecord<T> = CommonNodeRecord<VariableSizeNodeData<T>, T> & {
    height: number;
    data: VariableSizeNodeData<T>;
    readonly resize: (height: number, shouldForceUpdate?: boolean) => void;
};
export declare type VariableSizeTreeProps<T> = TreeProps<VariableSizeNodeData<T>, T> & Omit<VariableSizeListProps, 'children' | 'itemCount' | 'itemSize'> & {
    readonly children: React.ComponentType<VariableSizeNodeComponentProps<T>>;
    readonly itemSize?: VariableSizeListProps['itemSize'];
};
export declare type VariableSizeTreeState<T> = TreeState<VariableSizeNodeComponentProps<T>, VariableSizeNodeRecord<T>, VariableSizeNodeData<T>, T>;
export default class VariableSizeTree<T> extends React.PureComponent<VariableSizeTreeProps<T>, VariableSizeTreeState<T>> {
    static defaultProps: Partial<VariableSizeTreeProps<{}>>;
    static getDerivedStateFromProps({ children: component, itemData: treeData, }: VariableSizeTreeProps<{}>): Partial<VariableSizeTreeState<{}>>;
    private readonly list;
    constructor(props: VariableSizeTreeProps<T>, context: any);
    recomputeTree(options?: VariableSizeUpdateOptions): Promise<void>;
    resetAfterId(id: string | symbol, shouldForceUpdate?: boolean): void;
    scrollTo(scrollOffset: number): void;
    scrollToItem(id: string | symbol, align?: Align): void;
    render(): React.ReactNode;
    private computeTree;
    private createNodeRecord;
    private getItemSize;
}
