import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/extends";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import * as React from 'react';
import { FixedSizeList } from 'react-window';
import { Row } from './utils';

var FixedSizeTree =
/*#__PURE__*/
function (_React$PureComponent) {
  _inheritsLoose(FixedSizeTree, _React$PureComponent);

  FixedSizeTree.getDerivedStateFromProps = function getDerivedStateFromProps(_ref) {
    var component = _ref.children,
        treeData = _ref.itemData;
    return {
      component: component,
      treeData: treeData
    };
  };

  function FixedSizeTree(props, context) {
    var _this;

    _this = _React$PureComponent.call(this, props, context) || this;
    _this.list = React.createRef();
    _this.createNodeRecord = _this.createNodeRecord.bind(_assertThisInitialized(_this));
    var initialState = {
      component: props.children,
      order: [],
      records: {}
    };
    _this.state = _extends({}, initialState, {}, _this.computeTree({
      refreshNodes: true
    }, props, initialState));
    return _this;
  }

  var _proto = FixedSizeTree.prototype;

  _proto.recomputeTree = function recomputeTree(options) {
    try {
      var _this3 = this;

      return Promise.resolve(new Promise(function (resolve) {
        _this3.setState(function (prevState) {
          return _this3.computeTree(options, _this3.props, prevState);
        }, resolve);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.scrollTo = function scrollTo(scrollOffset) {
    var _this$list$current;

    (_this$list$current = this.list.current) == null ? void 0 : _this$list$current.scrollTo(scrollOffset);
  };

  _proto.scrollToItem = function scrollToItem(id, align) {
    var _this$list$current2;

    (_this$list$current2 = this.list.current) == null ? void 0 : _this$list$current2.scrollToItem(this.state.order.indexOf(id) || 0, align);
  };

  _proto.render = function render() {
    var _this$props = this.props,
        children = _this$props.children,
        treeWalker = _this$props.treeWalker,
        rowComponent = _this$props.rowComponent,
        rest = _objectWithoutPropertiesLoose(_this$props, ["children", "treeWalker", "rowComponent"]);

    return React.createElement(FixedSizeList, Object.assign({}, rest, {
      itemData: this.state,
      itemCount: this.state.order.length,
      ref: this.list
    }), rowComponent);
  };

  _proto.computeTree = function computeTree(_temp, _ref2, _ref3) {
    var _ref4 = _temp === void 0 ? {} : _temp,
        _ref4$refreshNodes = _ref4.refreshNodes,
        refreshNodes = _ref4$refreshNodes === void 0 ? false : _ref4$refreshNodes,
        _ref4$useDefaultOpenn = _ref4.useDefaultOpenness,
        useDefaultOpenness = _ref4$useDefaultOpenn === void 0 ? false : _ref4$useDefaultOpenn;

    var treeWalker = _ref2.treeWalker;
    var prevRecords = _ref3.records;
    var order = [];

    var records = _extends({}, prevRecords);

    var iter = treeWalker(refreshNodes);

    if (useDefaultOpenness) {
      for (var id in records) {
        records[id].isOpen = records[id].data.isOpenByDefault;
      }
    }

    var isPreviousOpened = false;

    while (true) {
      var _iter$next = iter.next(isPreviousOpened),
          done = _iter$next.done,
          value = _iter$next.value;

      if (done || !value) {
        break;
      }

      var _id = void 0;

      if (typeof value === 'string' || typeof value === 'symbol') {
        _id = value;

        if (useDefaultOpenness) {
          records[_id].isOpen = records[_id].data.isOpenByDefault;
        }
      } else {
        _id = value.id;
        var isOpenByDefault = value.isOpenByDefault;
        var record = records[_id];

        if (!record) {
          records[_id] = this.createNodeRecord(value);
        } else {
          record.data = value;

          if (useDefaultOpenness) {
            record.isOpen = isOpenByDefault;
          }
        }
      }

      order.push(_id);
      isPreviousOpened = records[_id].isOpen;
    }

    return {
      order: order,
      records: records
    };
  };

  _proto.createNodeRecord = function createNodeRecord(data) {
    var _this4 = this;

    var record = {
      data: data,
      isOpen: data.isOpenByDefault,
      toggle: function () {
        try {
          record.isOpen = !record.isOpen;
          return Promise.resolve(_this4.recomputeTree({
            refreshNodes: record.isOpen
          })).then(function () {});
        } catch (e) {
          return Promise.reject(e);
        }
      }
    };
    return record;
  };

  return FixedSizeTree;
}(React.PureComponent);

FixedSizeTree.defaultProps = {
  rowComponent: Row
};
export { FixedSizeTree as default };