import * as React from 'react';
import { FixedSizeList } from 'react-window';
import { Row } from './utils';
export default class FixedSizeTree extends React.PureComponent {
  static getDerivedStateFromProps({
    children: component,
    itemData: treeData
  }) {
    return {
      component,
      treeData
    };
  }

  constructor(props, context) {
    super(props, context);
    this.list = React.createRef();
    this.createNodeRecord = this.createNodeRecord.bind(this);
    const initialState = {
      component: props.children,
      order: [],
      records: {}
    };
    this.state = { ...initialState,
      ...this.computeTree({
        refreshNodes: true
      }, props, initialState)
    };
  }

  async recomputeTree(options) {
    return new Promise(resolve => {
      this.setState(prevState => this.computeTree(options, this.props, prevState), resolve);
    });
  }

  scrollTo(scrollOffset) {
    var _this$list$current;

    (_this$list$current = this.list.current) == null ? void 0 : _this$list$current.scrollTo(scrollOffset);
  }

  scrollToItem(id, align) {
    var _this$list$current2;

    (_this$list$current2 = this.list.current) == null ? void 0 : _this$list$current2.scrollToItem(this.state.order.indexOf(id) || 0, align);
  }

  render() {
    const {
      children,
      treeWalker,
      rowComponent,
      ...rest
    } = this.props;
    return React.createElement(FixedSizeList, Object.assign({}, rest, {
      itemData: this.state,
      itemCount: this.state.order.length,
      ref: this.list
    }), rowComponent);
  }

  computeTree({
    refreshNodes = false,
    useDefaultOpenness = false
  } = {}, {
    treeWalker
  }, {
    records: prevRecords
  }) {
    const order = [];
    const records = { ...prevRecords
    };
    const iter = treeWalker(refreshNodes);

    if (useDefaultOpenness) {
      for (const id in records) {
        records[id].isOpen = records[id].data.isOpenByDefault;
      }
    }

    let isPreviousOpened = false;

    while (true) {
      const {
        done,
        value
      } = iter.next(isPreviousOpened);

      if (done || !value) {
        break;
      }

      let id;

      if (typeof value === 'string' || typeof value === 'symbol') {
        id = value;

        if (useDefaultOpenness) {
          records[id].isOpen = records[id].data.isOpenByDefault;
        }
      } else {
        ({
          id
        } = value);
        const {
          isOpenByDefault
        } = value;
        const record = records[id];

        if (!record) {
          records[id] = this.createNodeRecord(value);
        } else {
          record.data = value;

          if (useDefaultOpenness) {
            record.isOpen = isOpenByDefault;
          }
        }
      }

      order.push(id);
      isPreviousOpened = records[id].isOpen;
    }

    return {
      order,
      records
    };
  }

  createNodeRecord(data) {
    const record = {
      data,
      isOpen: data.isOpenByDefault,
      toggle: async () => {
        record.isOpen = !record.isOpen;
        await this.recomputeTree({
          refreshNodes: record.isOpen
        });
      }
    };
    return record;
  }

}
FixedSizeTree.defaultProps = {
  rowComponent: Row
};