import * as React from 'react';
import { VariableSizeList } from 'react-window';
import { Row } from './utils';
export default class VariableSizeTree extends React.PureComponent {
  static getDerivedStateFromProps({
    children: component,
    itemData: treeData
  }) {
    return {
      component,
      treeData
    };
  }

  constructor(props, context) {
    super(props, context);
    this.list = React.createRef();
    this.getItemSize = this.getItemSize.bind(this);
    const initialState = {
      component: props.children,
      order: [],
      records: {}
    };
    this.state = { ...initialState,
      ...this.computeTree({
        refreshNodes: true
      }, props, initialState)
    };
  }

  async recomputeTree(options) {
    return new Promise(resolve => {
      this.setState(prevState => this.computeTree(options, this.props, prevState), () => {
        if (options == null ? void 0 : options.useDefaultHeight) {
          var _this$list$current;

          (_this$list$current = this.list.current) == null ? void 0 : _this$list$current.resetAfterIndex(0, true);
        }

        resolve();
      });
    });
  }

  resetAfterId(id, shouldForceUpdate = false) {
    var _this$list$current2;

    (_this$list$current2 = this.list.current) == null ? void 0 : _this$list$current2.resetAfterIndex(this.state.order.indexOf(id), shouldForceUpdate);
  }

  scrollTo(scrollOffset) {
    var _this$list$current3;

    (_this$list$current3 = this.list.current) == null ? void 0 : _this$list$current3.scrollTo(scrollOffset);
  }

  scrollToItem(id, align) {
    var _this$list$current4;

    (_this$list$current4 = this.list.current) == null ? void 0 : _this$list$current4.scrollToItem(this.state.order.indexOf(id) || 0, align);
  }

  render() {
    const {
      children,
      itemSize,
      rowComponent,
      treeWalker,
      ...rest
    } = this.props;
    return React.createElement(VariableSizeList, Object.assign({}, rest, {
      itemData: this.state,
      itemCount: this.state.order.length // tslint:disable-next-line:no-unbound-method
      ,
      itemSize: itemSize || this.getItemSize,
      ref: this.list
    }), rowComponent);
  }

  computeTree({
    refreshNodes = false,
    useDefaultHeight = false,
    useDefaultOpenness = false
  } = {}, {
    treeWalker
  }, {
    records: prevRecords
  }) {
    const order = [];
    const records = { ...prevRecords
    };
    const iter = treeWalker(refreshNodes);

    if (useDefaultHeight || useDefaultOpenness) {
      for (const id in records) {
        if (useDefaultHeight) {
          records[id].height = records[id].data.defaultHeight;
        }

        if (useDefaultOpenness) {
          records[id].isOpen = records[id].data.isOpenByDefault;
        }
      }
    }

    let isPreviousOpened = false;

    while (true) {
      const {
        done,
        value
      } = iter.next(isPreviousOpened);

      if (done || !value) {
        break;
      }

      let id;

      if (typeof value === 'string' || typeof value === 'symbol') {
        id = value;

        if (useDefaultOpenness) {
          records[id].isOpen = records[id].data.isOpenByDefault;
        }

        if (useDefaultHeight) {
          records[id].height = records[id].data.defaultHeight;
        }
      } else {
        ({
          id
        } = value);
        const {
          defaultHeight,
          isOpenByDefault
        } = value;
        const record = records[id];

        if (!record) {
          records[id] = this.createNodeRecord(value);
        } else {
          record.data = value;

          if (useDefaultOpenness) {
            record.isOpen = isOpenByDefault;
          }

          if (useDefaultHeight) {
            record.height = defaultHeight;
          }
        }
      }

      order.push(id);
      isPreviousOpened = records[id].isOpen;
    }

    return {
      order,
      records
    };
  }

  createNodeRecord(data) {
    const record = {
      data,
      height: data.defaultHeight,
      isOpen: data.isOpenByDefault,
      resize: (height, shouldForceUpdate) => {
        record.height = height;
        this.resetAfterId(record.data.id, shouldForceUpdate);
      },
      toggle: async () => {
        record.isOpen = !record.isOpen;
        await this.recomputeTree({
          refreshNodes: record.isOpen,
          useDefaultHeight: true
        });
      }
    };
    return record;
  }

  getItemSize(index) {
    const {
      order,
      records
    } = this.state;
    return records[order[index]].height;
  }

}
VariableSizeTree.defaultProps = {
  rowComponent: Row
};